@@ -32,6 +32,9 @@ import pytorch_lightning as pl
 import torch
 from PIL import Image
 from pytorch_lightning.utilities import rank_zero_only
+from pytorch_segmentation_models_trainer.custom_models.mod_polymapper.modpolymapper import (
+    GenericModPolyMapper,
+)
 from pytorch_segmentation_models_trainer.tools.visualization.base_plot_tools import (
     batch_denormalize_tensor,
     denormalize_np_array,
@@ -46,7 +49,7 @@ from pytorch_segmentation_models_trainer.tools.visualization.crossfield_plot imp
 from pytorch_segmentation_models_trainer.utils import polygonrnn_utils
 
 
-class ImageSegmentationResultCallback(pl.callbacks.base.Callback):
+class ImageSegmentationResultCallback(pl.callbacks.Callback):
     def __init__(
         self,
         n_samples: int = None,
@@ -148,48 +151,47 @@ class FrameFieldResultCallback(ImageSegmentationResultCallback):
         val_ds = pl_module.val_dataloader()
         device = pl_module.device
         logger = trainer.logger
-        batch = next(iter(val_ds))
-        image_display = batch_denormalize_tensor(batch["image"]).to("cpu")
-        pred = pl_module(batch["image"].to(device))
-        self.n_samples = (
+        n_samples = (
             pl_module.val_dataloader().batch_size
             if self.n_samples is None
             else self.n_samples
         )
-        for i in range(self.n_samples):
-            image = image_display[i].numpy()
-            mask = batch["gt_polygons_image"][i]
-            predicted_mask = pred["seg"][i]
-            predicted_mask = predicted_mask.to("cpu")
-            plot_title = val_ds.dataset.get_path(i)
-            crossfield = pred["crossfield"]
-            np_crossfield = crossfield.cpu().detach().numpy().transpose(0, 2, 3, 1)
-            image_to_plot = np.transpose(image, (1, 2, 0))
-            # image_plot_crossfield_list = [
-            #     get_image_plot_crossfield(
-            #         _crossfield, crossfield_stride=10, width=0.2
-            #     )
-            #     for _crossfield in np_crossfield
-            # ]
-            axarr, fig = generate_visualization(
-                fig_title=plot_title,
-                image=image_to_plot,
-                ground_truth_mask=self.prepare_mask_to_plot(mask.numpy()[0]),
-                predicted_mask=self.prepare_mask_to_plot(predicted_mask.numpy()[0]),
-                ground_truth_boundary=self.prepare_mask_to_plot(mask.numpy()[1]),
-                predicted_boundary=self.prepare_mask_to_plot(predicted_mask.numpy()[1]),
-                # crossfield=image_plot_crossfield_list[i],
-            )
-            fig.tight_layout()
-            fig.subplots_adjust(top=0.95)
-            if self.save_outputs:
-                saved_image = self.save_plot_to_disk(
-                    plt, plot_title, trainer.current_epoch
+        current_item = 0
+        for batch in val_ds:
+            if current_item >= n_samples:
+                break
+            images = batch["image"]
+            image_display = batch_denormalize_tensor(images).to("cpu")
+            pred = pl_module(images.to(device))
+            for i in range(images.shape[0]):
+                if current_item >= n_samples:
+                    break
+                mask = batch["gt_polygons_image"][i]
+                predicted_mask = pred["seg"][i]
+                predicted_mask = predicted_mask.to("cpu")
+                plot_title = batch["path"][i]
+                image_to_plot = np.transpose(image_display[i], (1, 2, 0))
+                axarr, fig = generate_visualization(
+                    fig_title=plot_title,
+                    image=image_to_plot,
+                    ground_truth_mask=self.prepare_mask_to_plot(mask.numpy()[0]),
+                    predicted_mask=self.prepare_mask_to_plot(predicted_mask.numpy()[0]),
+                    ground_truth_boundary=self.prepare_mask_to_plot(mask.numpy()[1]),
+                    predicted_boundary=self.prepare_mask_to_plot(
+                        predicted_mask.numpy()[1]
+                    ),
                 )
-                self.log_data_to_tensorboard(
-                    saved_image, plot_title, logger, trainer.current_epoch
-                )
-            plt.close(fig)
+                fig.tight_layout()
+                fig.subplots_adjust(top=0.95)
+                if self.save_outputs:
+                    saved_image = self.save_plot_to_disk(
+                        plt, plot_title, trainer.current_epoch
+                    )
+                    self.log_data_to_tensorboard(
+                        saved_image, plot_title, logger, trainer.current_epoch
+                    )
+                plt.close(fig)
+                current_item += 1
         return
 
 
@@ -342,13 +344,15 @@ class PolygonRNNResultCallback(ImageSegmentationResultCallback):
                 pl_module.val_seq_len,
             )
             gt_polygon_list = prepared_item["shapely_polygon_list"]
-            predicted_polygon_list = polygonrnn_utils.get_vertex_list_from_batch_tensors(
-                output_batch_polygons,
-                prepared_item["scale_h"],
-                prepared_item["scale_w"],
-                prepared_item["min_col"],
-                prepared_item["min_row"],
-                grid_size=val_ds.dataset.grid_size,
+            predicted_polygon_list = (
+                polygonrnn_utils.get_vertex_list_from_batch_tensors(
+                    output_batch_polygons,
+                    prepared_item["scale_h"],
+                    prepared_item["scale_w"],
+                    prepared_item["min_col"],
+                    prepared_item["min_row"],
+                    grid_size=val_ds.dataset.grid_size,
+                )
             )
             saved_image = self.build_polygon_vis(
                 image_path,
@@ -380,20 +384,17 @@ class ModPolyMapperResultCallback(PolygonRNNResultCallback):
         )
         self.threshold = threshold
         self.show_label_scores = show_label_scores
+        self.n_samples = 16 if n_samples is None else n_samples
 
     @rank_zero_only
     def on_validation_end(self, trainer: pl.Trainer, pl_module: pl.LightningModule):
         val_ds = pl_module.val_dataloader().loaders  # type: ignore
-        self.n_samples = (
-            val_ds["object_detection"].loader.batch_size
-            if self.n_samples is None
-            else self.n_samples
-        )
         current_item = 0
         prepared_input = val_ds[
             "polygon_rnn"
         ].loader.dataset.get_n_image_path_dict_list(self.n_samples)
-        model = pl_module.model
+        model: GenericModPolyMapper = pl_module.model  # type: ignore
+        grid_size = model.polygonrnn_model.grid_size
         model.eval()  # type: ignore
         aug_func = A.Compose([A.Normalize(), ToTensorV2()])
         for image_path, prepared_item in prepared_input.items():
@@ -411,6 +412,7 @@ class ModPolyMapperResultCallback(PolygonRNNResultCallback):
                 image_path=image_path,
                 prepared_item=prepared_item,
                 detection_dict=outputs[0],
+                grid_size=grid_size,
             )
             current_item += 1
         return
@@ -421,6 +423,7 @@ class ModPolyMapperResultCallback(PolygonRNNResultCallback):
         image_path: str,
         prepared_item: Dict[str, torch.Tensor],
         detection_dict: Dict[str, torch.Tensor],
+        grid_size: int,
     ) -> None:
         gt_polygon_list = prepared_item["shapely_polygon_list"]
         predicted_polygon_list = polygonrnn_utils.get_vertex_list_from_batch_tensors(
@@ -429,6 +432,7 @@ class ModPolyMapperResultCallback(PolygonRNNResultCallback):
             scale_w=detection_dict["scale_w"],
             min_row=detection_dict["min_row"],
             min_col=detection_dict["min_col"],
+            grid_size=grid_size,
         )
         self.build_obj_det_and_polygon_vis(
             image_path=image_path,
