@@ -26,6 +26,10 @@ from pytorch_segmentation_models_trainer.train import train
 from pytorch_segmentation_models_trainer.build_mask import build_masks
 from pytorch_segmentation_models_trainer.config_utils import validate_config
 from pytorch_segmentation_models_trainer.convert_ds import convert_dataset
+from pytorch_segmentation_models_trainer.predict_from_batch import predict_from_batch
+from pytorch_segmentation_models_trainer.predict_mod_polymapper_from_batch import (
+    predict_mod_polymapper_from_batch,
+)
 
 import logging
 import warnings
@@ -33,8 +37,12 @@ from rasterio.errors import NotGeoreferencedWarning
 
 logging.getLogger("shapely.geos").setLevel(logging.CRITICAL)
 logging.getLogger("rasterio.errors").setLevel(logging.CRITICAL)
+logging.getLogger("tensorboard").setLevel(logging.CRITICAL)
+logging.getLogger("numpy").setLevel(logging.CRITICAL)
+logging.getLogger("skan").setLevel(logging.CRITICAL)
 warnings.filterwarnings("ignore", category=NotGeoreferencedWarning)
 warnings.simplefilter(action="ignore", category=Warning)
+warnings.filterwarnings("ignore", category=DeprecationWarning)
 
 
 @hydra.main(config_path="conf")
@@ -43,6 +51,10 @@ def main(cfg: DictConfig):
         return train(cfg)
     elif cfg.mode == "predict":
         return predict(cfg)
+    elif cfg.mode == "predict-from-batch":
+        return predict_from_batch(cfg)
+    elif cfg.mode == "predict-mod-polymapper-from-batch":
+        return predict_mod_polymapper_from_batch(cfg)
     elif cfg.mode == "validate-config":
         return validate_config(cfg)
     elif cfg.mode == "build-mask":
