@@ -25,6 +25,7 @@ from collections import defaultdict
 from typing import Callable, List, Optional, Union, Tuple, Dict
 from PIL import Image, ImageDraw
 import numpy as np
+from shapely.geometry.collection import GeometryCollection
 import torch
 import itertools
 from shapely.geometry import Polygon, LineString, Point, MultiPolygon, box
@@ -110,7 +111,7 @@ def get_vertex_list_from_numpy(
         return x.cpu().numpy() if isinstance(x, torch.Tensor) else x
 
     input_array = cast_to_np(input_array)
-    if np.max(input_array) >= grid_size ** 2:
+    if np.max(input_array) >= grid_size**2:
         length = np.argmax(input_array)
         input_array = input_array[:length]
     if input_array.shape[0] == 0:
@@ -208,7 +209,9 @@ def get_vertex_list_from_batch_tensors(
     def cast_func(x):
         return np.array(x, dtype=np.float32)
 
-    if input_batch == []:
+    if input_batch == [] or (
+        isinstance(input_batch, torch.Tensor) and input_batch.numel() == 0
+    ):
         return []
 
     return [
@@ -379,7 +382,7 @@ def validate_polygon(geom: Polygon) -> List[Union[Polygon, MultiPolygon]]:
     valid_output = make_valid(geom)
     if isinstance(valid_output, (Polygon, MultiPolygon)):
         return [valid_output]
-    if isinstance(valid_output, list):
+    if isinstance(valid_output, (list, GeometryCollection)):
         return [p for p in valid_output if isinstance(p, (Polygon, MultiPolygon))]
     else:
         return []
@@ -421,17 +424,17 @@ def get_scales(
     target_width: float = 224.0,
 ) -> tuple:
     """
-        Gets scales for the image.
+    Gets scales for the image.
 
-        Args:
-            min_row (int): min row
-            min_col (int): min col
-            max_row (int): max row
-            max_col (int): max col
+    Args:
+        min_row (int): min row
+        min_col (int): min col
+        max_row (int): max row
+        max_col (int): max col
 
-        Returns:
-            tuple: scale_h, scale_w
-        """
+    Returns:
+        tuple: scale_h, scale_w
+    """
     object_h = max_row - min_row
     object_w = max_col - min_col
     scale_h = target_height / object_h
